@@ -1,28 +1,24 @@
 package Data::Munge;
 
-use warnings;
 use strict;
+use warnings;
 use base qw(Exporter);
 
-our $VERSION = '0.08';
-our @EXPORT = qw[
-    list2re
-    byval
-    mapval
-    submatches
-    replace
-    eval_string
-    rec
-    trim
-    elem
-];
+sub _eval { eval $_[0] }  # empty lexical scope
 
-sub list2re {
-    @_ or return qr/(?!)/;
-    my $re = join '|', map quotemeta, sort {length $b <=> length $a || $a cmp $b } @_;
-    $re eq '' and $re = '(?#)';
-    qr/$re/
-}
+our $VERSION = '0.095';
+our @EXPORT = qw(
+    byval
+    elem
+    eval_string
+    list2re
+    mapval
+    rec
+    replace
+    slurp
+    submatches
+    trim
+);
 
 sub byval (&$) {
     my ($f, $x) = @_;
@@ -31,46 +27,6 @@ sub byval (&$) {
     $x
 }
 
-sub mapval (&@) {
-    my $f = shift;
-    my @xs = @_;
-    map { $f->($_); $_ } @xs
-}
-
-sub submatches {
-    no strict 'refs';
-    map $$_, 1 .. $#+
-}
-
-sub replace {
-    my ($str, $re, $x, $g) = @_;
-    my $f = ref $x ? $x : sub {
-        my $r = $x;
-        $r =~ s{\$([\$&`'0-9]|\{([0-9]+)\})}{
-            $+ eq '$' ? '$' :
-            $+ eq '&' ? $_[0] :
-            $+ eq '`' ? substr($_[-1], 0, $_[-2]) :
-            $+ eq "'" ? substr($_[-1], $_[-2] + length $_[0]) :
-            $_[$+]
-        }eg;
-        $r
-    };
-    if ($g) {
-        $str =~ s{$re}{ $f->(substr($str, $-[0], $+[0] - $-[0]), submatches, $-[0], $str) }eg;
-    } else {
-        $str =~ s{$re}{ $f->(substr($str, $-[0], $+[0] - $-[0]), submatches, $-[0], $str) }e;
-    }
-    $str
-}
-
-sub trim {
-    my ($s) = @_;
-    return undef if !defined $s;
-    $s =~ s/^\s+//;
-    $s =~ s/\s+\z//;
-    $s
-}
-
 sub elem {
     my ($k, $xs) = @_;
     if (ref $k) {
@@ -79,7 +35,7 @@ sub elem {
         }
     } elsif (defined $k) {
         for my $x (@$xs) {
-            return 1 if defined $x && $k eq $x;
+            return 1 if defined $x && !ref $x && $k eq $x;
         }
     } else {
         for my $x (@$xs) {
@@ -89,8 +45,6 @@ sub elem {
     !1
 }
 
-sub _eval { eval $_[0] }  # empty lexical scope
-
 sub eval_string {
     my ($code) = @_;
     my ($package, $file, $line) = caller;
@@ -100,6 +54,19 @@ sub eval_string {
     wantarray ? @r : $r[0]
 }
 
+sub list2re {
+    @_ or return qr/(?!)/;
+    my $re = join '|', map quotemeta, sort {length $b <=> length $a || $a cmp $b } @_;
+    $re eq '' and $re = '(?#)';
+    qr/$re/
+}
+
+sub mapval (&@) {
+    my $f = shift;
+    my @xs = @_;
+    map { $f->($_); $_ } @xs
+}
+
 if ($] >= 5.016) {
     eval_string <<'EOT';
 use v5.16;
@@ -124,6 +91,45 @@ EOT
     };
 }
 
+sub replace {
+    my ($str, $re, $x, $g) = @_;
+    my $f = ref $x ? $x : sub {
+        my $r = $x;
+        $r =~ s{\$([\$&`'0-9]|\{([0-9]+)\})}{
+            $+ eq '$' ? '$' :
+            $+ eq '&' ? $_[0] :
+            $+ eq '`' ? substr($_[-1], 0, $_[-2]) :
+            $+ eq "'" ? substr($_[-1], $_[-2] + length $_[0]) :
+            $_[$+]
+        }eg;
+        $r
+    };
+    if ($g) {
+        $str =~ s{$re}{ $f->(substr($str, $-[0], $+[0] - $-[0]), submatches(), $-[0], $str) }eg;
+    } else {
+        $str =~ s{$re}{ $f->(substr($str, $-[0], $+[0] - $-[0]), submatches(), $-[0], $str) }e;
+    }
+    $str
+}
+
+sub slurp {
+    local $/;
+    scalar readline $_[0]
+}
+
+sub submatches {
+    no strict 'refs';
+    map $$_, 1 .. $#+
+}
+
+sub trim {
+    my ($s) = @_;
+    return undef if !defined $s;
+    $s =~ s/^\s+//;
+    $s =~ s/\s+\z//;
+    $s
+}
+
 'ok'
 
 __END__
@@ -136,19 +142,27 @@ Data::Munge - various utility functions
 
  use Data::Munge;
  
- my $re = list2re qw/foo bar baz/;
+ my $re = list2re qw/f ba foo bar baz/;
+ # $re = qr/bar|baz|foo|ba|f/;
  
  print byval { s/foo/bar/ } $text;
+ # print do { my $tmp = $text; $tmp =~ s/foo/bar/; $tmp };
+ 
  foo(mapval { chomp } @lines);
+ # foo(map { my $tmp = $_; chomp $tmp; $tmp } @lines);
  
  print replace('Apples are round, and apples are juicy.', qr/apples/i, 'oranges', 'g');
+ # "oranges are round, and oranges are juicy."
  print replace('John Smith', qr/(\w+)\s+(\w+)/, '$2, $1');
+ # "Smith, John"
  
  my $trimmed = trim "  a b c "; # "a b c"
  
  my $x = 'bar';
  if (elem $x, [qw(foo bar baz)]) { ... }
  
+ my $contents = slurp $fh;  # or: slurp *STDIN
+ 
  eval_string('print "hello world\\n"');  # says hello
  eval_string('die');  # dies
  eval_string('{');    # throws a syntax error
@@ -158,6 +172,11 @@ Data::Munge - various utility functions
    $n < 2 ? 1 : $n * $rec->($n - 1)
  };
  print $fac->(5);  # 120
+ 
+ if ("hello, world!" =~ /(\w+), (\w+)/) {
+   my @captured = submatches;
+   # @captured = ("hello", "world")
+ }
 
 =head1 DESCRIPTION
 
@@ -166,7 +185,7 @@ redefining or working around them, so I wrote this module.
 
 =head2 Functions
 
-=over 4
+=over
 
 =item list2re LIST
 
@@ -176,6 +195,29 @@ Especially useful in combination with C<keys>. Example:
  my $re = list2re keys %hash;
  $str =~ s/($re)/$hash{$1}/g;
 
+This function takes special care to get several edge cases right:
+
+=over
+
+=item *
+
+Empty list: An empty argument list results in a regex that doesn't match
+anything.
+
+=item *
+
+Empty string: An argument list consisting of a single empty string results in a
+regex that matches the empty string (and nothing else).
+
+=item *
+
+Prefixes: The input strings are sorted by descending length to ensure longer
+matches are tried before shorter matches. Otherwise C<list2re('ab', 'abcd')>
+would generate C<qr/ab|abcd/>, which (on its own) can never match C<abcd>
+(because C<ab> is tried first, and it always succeeds where C<abcd> could).
+
+=back
+
 =item byval BLOCK SCALAR
 
 Takes a code block and a value, runs the block with C<$_> set to that value,
@@ -187,6 +229,12 @@ in the block will not affect the passed in value. Example:
  # Calls foo() with the value of $str, but all '!' have been replaced by '?'.
  # $str itself is not modified.
 
+Since perl 5.14 you can also use the C</r> flag:
+
+ foo($str =~ s/!/?/gr);
+
+But C<byval> works on all versions of perl and is not limited to C<s///>.
+
 =item mapval BLOCK LIST
 
 Works like a combination of C<map> and C<byval>; i.e. it behaves like
@@ -274,7 +322,23 @@ at elements C<1 .. 9999>).
 
 =item eval_string STRING
 
-Evals I<STRING> just like C<eval> but doesn't catch exceptions.
+Evals I<STRING> just like C<eval> but doesn't catch exceptions. Caveat: Unlike
+with C<eval> the code runs in an empty lexical scope:
+
+ my $foo = "Hello, world!\n";
+ eval_string 'print $foo';
+ # Dies: Global symbol "$foo" requires explicit package name
+
+That is, the eval'd code can't see variables from the scope of the
+C<eval_string> call.
+
+=item slurp FILEHANDLE
+
+Reads and returns all remaining data from I<FILEHANDLE> as a string, or
+C<undef> if it hits end-of-file. (Interaction with non-blocking filehandles is
+currently not well defined.)
+
+C<slurp $handle> is equivalent to C<do { local $/; scalar readline $handle }>.
 
 =item rec BLOCK
 
@@ -313,7 +377,7 @@ Lukas Mai, C<< <l.mai at web.de> >>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2011, 2013-2014 Lukas Mai.
+Copyright 2009-2011, 2013-2015 Lukas Mai.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
